@@ -32,7 +32,7 @@
 extern int do_transparent_proxy (struct conn_s *connptr,
                                  hashmap_t hashofheaders,
                                  struct request_s *request,
-                                 struct config_s *config, char *url);
+                                 struct config_s *config, char **url);
 
 #endif
 
